@@ -3,7 +3,7 @@ from __future__ import annotations
 import inspect
 from datetime import datetime, timedelta
 from pathlib import Path
-from typing import Any, Dict, Iterable, List, Optional, Tuple, Union
+from typing import Any, Dict, Iterable, List, Optional, Tuple, Union, overload
 
 import python_on_whales.components.image
 import python_on_whales.components.network
@@ -55,7 +55,26 @@ class ContainerHostConfig(DockerCamelModel):
     privileged: bool
     publish_all_ports: bool
     readonly_rootfs: bool
+    shm_size: int
+    runtime: str
+    console_size: List[int]
+    cpu_shares: int
+    memory: int
+    nano_cpus: int
+    cgroup_parent: str
+    blkio_weight: int
+    cpu_period: int
+    cpu_quota: int
+    cpu_realtime_period: int
+    cpu_realtime_runtime: int
+    kernel_memory: int
+    memory_reservation: int
+    memory_swap: int
+    oom_kill_disable: bool
     cpu_count: float
+    cpu_percent: float
+    masked_paths: Optional[List[Path]]
+    readonly_paths: Optional[List[Path]]
 
 
 class ContainerConfig(DockerCamelModel):
@@ -213,6 +232,8 @@ class Container(ReloadableObjectFromJson):
     ) -> python_on_whales.components.image.Image:
         """Create a new image from the container's changes.
 
+        Alias: `docker.commit(...)`
+
         See the [`docker.container.commit`](../sub-commands/container.md) command for
         information about the arguments.
         """
@@ -283,6 +304,14 @@ class Container(ReloadableObjectFromJson):
         """
         return ContainerCLI(self.client_config).pause(self)
 
+    def unpause(self) -> None:
+        """Unpause the container
+
+        See the [`docker.container.unpause`](../sub-commands/container.md#unpause) command for
+        information about the arguments.
+        """
+        return ContainerCLI(self.client_config).unpause(self)
+
     def rename(self, new_name: str) -> None:
         """Rename this container
 
@@ -386,10 +415,12 @@ class ContainerCLI(DockerCLICaller):
     ):
         """Copy files/folders between a container and the local filesystem
 
+        Alias: `docker.copy(...)`
+
         ```python
         from python_on_whales import docker
 
-        docker.run("ubuntu", ["sleep", "infinity"], name="dodo", remove=True)
+        docker.run("ubuntu", ["sleep", "infinity"], name="dodo", remove=True, detach=True)
 
         docker.copy("/tmp/my_local_file.txt", ("dodo", "/path/in/container.txt"))
         docker.copy(("dodo", "/path/in/container.txt"), "/tmp/my_local_file2.txt")
@@ -523,6 +554,8 @@ class ContainerCLI(DockerCLICaller):
     ) -> Container:
         """Creates a container, but does not start it.
 
+        Alias: `docker.create(...)`
+
         Start it then with the `.start()` method.
 
         It might be useful if you want to delay the start of a container,
@@ -688,6 +721,8 @@ class ContainerCLI(DockerCLICaller):
     def diff(self, container: ValidContainer) -> Dict[str, str]:
         """List all the files modified, added or deleted since the container started.
 
+        Alias: `docker.diff(...)`
+
         # Arguments
             container: The container to inspect
 
@@ -712,6 +747,8 @@ class ContainerCLI(DockerCLICaller):
     ) -> Optional[str]:
         """Execute a command inside a container
 
+        Alias: `docker.execute(...)`
+
         # Arguments
             container: The container to execute the command in.
             command: The command to execute.
@@ -738,6 +775,8 @@ class ContainerCLI(DockerCLICaller):
     def export(self, container: ValidContainer, output: ValidPath) -> None:
         """Export a container's filesystem as a tar archive
 
+        Alias: `docker.export(...)`
+
         # Arguments
             container: The container to export.
             output: The path of the output tar archive.
@@ -751,7 +790,15 @@ class ContainerCLI(DockerCLICaller):
         ]
         run(full_cmd)
 
-    def inspect(self, x: Union[str, List[str]]) -> Container:
+    @overload
+    def inspect(self, x: str) -> Container:
+        ...
+
+    @overload
+    def inspect(self, x: List[str]) -> List[Container]:
+        ...
+
+    def inspect(self, x: Union[str, List[str]]) -> Union[Container, List[Container]]:
         """Returns a container object from a name or ID.
 
         # Arguments
@@ -774,6 +821,8 @@ class ContainerCLI(DockerCLICaller):
     ):
         """Kill a container.
 
+        Alias: `docker.kill(...)`
+
         # Arguments
             containers: One or more containers to kill
             signal: The signal to send the container
@@ -798,6 +847,10 @@ class ContainerCLI(DockerCLICaller):
     ) -> str:
         """Returns the logs of a container as a string.
 
+        Alias: `docker.logs(...)`
+
+        The follow option is not yet implemented.
+
         # Arguments
             container: The container to get the logs of
             details: Show extra details provided to logs
@@ -822,6 +875,8 @@ class ContainerCLI(DockerCLICaller):
     def list(self, all: bool = False, filters: Dict[str, str] = {}) -> List[Container]:
         """List the containers on the host.
 
+        Alias: `docker.ps(...)`
+
         # Arguments
             all: If `True`, also returns containers that are not running.
 
@@ -843,6 +898,8 @@ class ContainerCLI(DockerCLICaller):
     def pause(self, containers: Union[ValidContainer, List[ValidContainer]]):
         """Pauses one or more containers
 
+        Alias: `docker.pause(...)`
+
         # Arguments
             containers: One or more containers to pause
         """
@@ -866,6 +923,8 @@ class ContainerCLI(DockerCLICaller):
     def rename(self, container: ValidContainer, new_name: str) -> None:
         """Changes the name of a container.
 
+        Alias: `docker.rename(...)`
+
         # Arguments
             container: The container to rename
             new_name: The new name of the container.
@@ -880,6 +939,8 @@ class ContainerCLI(DockerCLICaller):
     ):
         """Restarts one or more container.
 
+        Alias: `docker.restart(...)`
+
         # Arguments
             containers: One or more containers to restart
             time: Amount of to wait for stop before killing the container (default 10s).
@@ -905,6 +966,8 @@ class ContainerCLI(DockerCLICaller):
     ) -> None:
         """Removes a container
 
+        Alias: `docker.remove(...)`
+
         # Arguments
             containers: One or more containers.
             force: Force the removal of a running container (uses SIGKILL)
@@ -1024,6 +1087,10 @@ class ContainerCLI(DockerCLICaller):
         For a deeper dive into the arguments and what they do, visit
         <https://docs.docker.com/engine/reference/run/>
 
+        If you want to know exactly how to call `docker.run()` depending on your
+        use case (detach, stream...), take a look at
+        the [`docker.run()` guide](../user_guide/docker_run.md).
+
         ```python
         >>> from python_on_whales import docker
         >>> returned_string = docker.run("hello-world")
@@ -1085,7 +1152,7 @@ class ContainerCLI(DockerCLICaller):
             command: List of arguments to provide to the container.
             add_hosts: hosts to add in the format of a tuple. For example,
                 `add_hosts=[("my_host_1", "192.168.30.31"), ("host2", "192.168.80.81")]`
-            blkio_weight:Block IO (relative weight), between 10 and 1000,
+            blkio_weight: Block IO (relative weight), between 10 and 1000,
                 or 0 to disable (default 0)
             cpu_period: Limit CPU CFS (Completely Fair Scheduler) period
             cpu_quota: Limit CPU CFS (Completely Fair Scheduler) quota
@@ -1094,6 +1161,8 @@ class ContainerCLI(DockerCLICaller):
             cpu_shares: CPU shares (relative weight)
             cpus: The maximal amount of cpu the container can use.
                 `1` means one cpu core.
+            cpuset_cpus: CPUs in which to allow execution. Must be given as a list.
+            cpuset_mems: MEMs in which to allow execution. Must be given as a list.
             detach: If `False`, returns the ouput of the container as a string.
                 If `True`, returns a `python_on_whales.Container` object.
             dns_search: Set custom DNS search domains
@@ -1352,7 +1421,14 @@ class ContainerCLI(DockerCLICaller):
             run(full_cmd)
 
     def stats(self):
-        """Not yet implemented"""
+        """Get containers resource usage statistics
+
+        Alias: `docker.stats(...)`
+
+        Not yet implemented
+        """
+        # We can do something nice with
+        # docker stats --format "{{json .}}" --no-stream --no-trunc
         raise NotImplementedError
 
     def stop(
@@ -1362,6 +1438,8 @@ class ContainerCLI(DockerCLICaller):
     ):
         """Stops one or more running containers
 
+        Alias: `docker.stop(...)`
+
         Aliases: `docker.stop`, `docker.container.stop`,
         `python_on_whales.Container.stop`.
 
@@ -1382,20 +1460,142 @@ class ContainerCLI(DockerCLICaller):
         run(full_cmd)
 
     def top(self):
-        """Not yet implemented"""
+        """Get the running processes of a container
+
+        Alias: `docker.top(...)`
+
+        Not yet implemented"""
         raise NotImplementedError
 
-    def unpause(self):
-        """Not yet implemented"""
-        raise NotImplementedError
+    def unpause(self, x: Union[ValidContainer, List[ValidContainer]]):
+        """Unpause all processes within one or more containers
 
-    def update(self):
-        """Not yet implemented"""
-        raise NotImplementedError
+        Alias: `docker.unpause(...)`
 
-    def wait(self):
-        """Not yet implemented"""
-        raise NotImplementedError
+        # Arguments
+            x: One or more containers (name, id or `python_on_whales.Container` object).
+        """
+        full_cmd = self.docker_cmd + ["container", "unpause"]
+        full_cmd += to_list(x)
+        run(full_cmd)
+
+    def update(
+        self,
+        x: Union[ValidContainer, List[ValidContainer]],
+        blkio_weight: Optional[int] = None,
+        cpu_period: Optional[int] = None,
+        cpu_quota: Optional[int] = None,
+        cpu_rt_period: Optional[int] = None,
+        cpu_rt_runtime: Optional[int] = None,
+        cpu_shares: Optional[int] = None,
+        cpus: Optional[float] = None,
+        cpuset_cpus: Optional[List[int]] = None,
+        cpuset_mems: Optional[List[int]] = None,
+        kernel_memory: Union[int, str, None] = None,
+        memory: Union[int, str, None] = None,
+        memory_reservation: Union[int, str, None] = None,
+        memory_swap: Union[int, str, None] = None,
+        pids_limit: Optional[int] = None,
+        restart: Optional[str] = None,
+    ):
+        """Update configuration of one or more containers
+
+        Alias: `docker.update(...)`
+
+        # Arguments
+            x: One or a list of containers to update.
+            blkio_weight: Block IO (relative weight), between 10 and 1000,
+                or 0 to disable (default 0)
+            cpu_period: Limit CPU CFS (Completely Fair Scheduler) period
+            cpu_quota: Limit CPU CFS (Completely Fair Scheduler) quota
+            cpu_rt_period: Limit CPU real-time period in microseconds
+            cpu_rt_runtime: Limit CPU real-time runtime in microseconds
+            cpu_shares: CPU shares (relative weight)
+            cpus: The maximal amount of cpu the container can use.
+                `1` means one cpu core.
+            cpuset_cpus: CPUs in which to allow execution. Must be given as a list.
+            cpuset_mems: MEMs in which to allow execution. Must be given as a list.
+            memory:  Memory limit, valid values are `1024` (ints are bytes) or
+                `"43m"` or `"6g"`.
+            memory_reservation: Memory soft limit
+            memory_swap: Swap limit equal to memory plus swap: '-1'
+                to enable unlimited swap.
+            pids_limit: Tune container pids limit (set `-1` for unlimited)
+            restart: Restart policy to apply when a container exits (default "no")
+        """
+        full_cmd = self.docker_cmd + ["container", "update"]
+        full_cmd.add_simple_arg("--blkio-weight", blkio_weight)
+        full_cmd.add_simple_arg("--cpu-period", cpu_period)
+        full_cmd.add_simple_arg("--cpu-quota", cpu_quota)
+        full_cmd.add_simple_arg("--cpu-rt-period", cpu_rt_period)
+        full_cmd.add_simple_arg("--cpu-rt-runtime", cpu_rt_runtime)
+        full_cmd.add_simple_arg("--cpu-shares", cpu_shares)
+        full_cmd.add_simple_arg("--cpus", cpus)
+        full_cmd.add_simple_arg("--cpuset-cpus", cpuset_cpus)
+        full_cmd.add_simple_arg("--cpuset-mems", cpuset_mems)
+        full_cmd.add_simple_arg("--kernel-memory", kernel_memory)
+        full_cmd.add_simple_arg("--memory", memory)
+        full_cmd.add_simple_arg("--memory-reservation", memory_reservation)
+        full_cmd.add_simple_arg("--memory-swap", memory_swap)
+        full_cmd.add_simple_arg("--pids-limit", pids_limit)
+        full_cmd.add_simple_arg("--restart", restart)
+        full_cmd += to_list(x)
+        run(full_cmd)
+
+    @overload
+    def wait(self, x: ValidContainer) -> int:
+        ...
+
+    @overload
+    def wait(self, x: List[ValidContainer]) -> List[int]:
+        ...
+
+    def wait(
+        self, x: Union[ValidContainer, List[ValidContainer]]
+    ) -> Union[int, List[int]]:
+        """Block until one or more containers stop, then returns their exit codes
+
+        Alias: `docker.wait(...)`
+
+
+        # Arguments
+            x: One or a list of containers to wait for.
+
+        # Returns
+            An `int` if a single container was passed as argument or a list of ints
+            if multiple containers were passed as arguments.
+
+        Some Examples:
+
+        ```python
+        cont = docker.run("ubuntu", ["bash", "-c", "sleep 2 && exit 8"], detach=True)
+
+        exit_code = docker.wait(cont)
+
+        print(exit_code)
+        # 8
+        docker.container.remove(cont)
+        ```
+
+        ```python
+        cont_1 = docker.run("ubuntu", ["bash", "-c", "sleep 4 && exit 8"], detach=True)
+        cont_2 = docker.run("ubuntu", ["bash", "-c", "sleep 2 && exit 10"], detach=True)
+
+        exit_codes = docker.wait([cont_1, cont_2])
+
+        print(exit_codes)
+        # [8, 10]
+        docker.container.remove([cont_1, cont_2])
+        ```
+        """
+        full_cmd = self.docker_cmd + ["container", "wait"]
+        if isinstance(x, list):
+            full_cmd += x
+            exit_codes = run(full_cmd)
+            return [int(exit_code) for exit_code in exit_codes.splitlines()]
+        else:
+            full_cmd.append(x)
+            return int(run(full_cmd))
 
 
 def format_time_for_docker(time_object: Union[datetime, timedelta]) -> str:
